// Copyright 2009, 2010 Christopher Smith
// 
// This file is part of smithers.cards.
// 
// smithers.cards is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.cards is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.cards. If not, see <http://www.gnu.org/licenses/>.

package smithers.cards;

/**
 * Simplifies the definition of a {@code BuildingRule} be redefining in terms
 * of a different collection of methods.
 */
public abstract class AbstractBuildingRule implements BuildingRule {
    
    /** Set to {@code true} if multiple cards can be added at once. */
    protected boolean canAddMultiple = true;
    
    /**
     * Determines whether card {@code c2} can be placed on top of card {@code
     * c1}.
     * 
     * @param c1 the card to move onto
     * @param c2 the card to be moved
     * @return {@code true} iff the card can be moved
     */
    public abstract boolean goesOn(Card c1, Card c2);
    
    /**
     * Determines whether the specified card can be placed in an empty space.
     * 
     * @param c the card to be moved
     * @return {@code true} iff the card can be moved into a space
     */
    public abstract boolean goesInSpace(Card c);
    
    /**
     * {@inheritDoc}
     */
    public boolean canAddCard(CardStack stack, Card add) {
        if (stack.getLength() == 0) return goesInSpace(add);
        return goesOn(stack.topCard(), add);
    }
    
    /**
     * {@inheritDoc}
     */
    public int canAddCards(CardStack stack, CardStack add) {
        if (canAddMultiple) {
            int i, addLength = add.getLength();
            for (i=1; i<=addLength; i++) {
                if (i>1 && ! goesOn(add.getCard(addLength-i), add.getCard(addLength-i+1))) return 0;
                if (stack.getLength() == 0) {
                    if (goesInSpace(add.getCard(addLength-i))) return i;
                } else {
                    if (goesOn(stack.topCard(), add.getCard(addLength-i))) return i;
                }
            }
            return 0;
        } else {
            return (add.getLength() > 0 && canAddCard(stack, add.topCard())) ? 1 : 0;
        }
    }
    
}
