// Copyright 2009 Christopher Smith
// 
// This file is part of smithers.extras.
// 
// smithers.extras is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.extras is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.extras. If not, see <http://www.gnu.org/licenses/>.

package smithers.extras;

import java.io.IOException;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JTextField;

/**
 * Swing-based test harness for the {@link smithers.extras.NumberString} class.
 */
public class NumberStringTest implements ActionListener {
    
    private long n = 123;
    
    // GUI Components
    /////////////////////////////
    //        jlbDigits        //
    //        jlbEnglish       //
    //        jlbFrench        //
    //        jlbGerman        //
    //         jlbRoman        //
    // jbtPlusOne  jbtMinusOne //
    // jbtPlusTen  jbtMinusTen //
    // jtfNewValue jbtSetValue //
    /////////////////////////////
    private JLabel jlbDigits, jlbEnglish, jlbFrench, jlbGerman, jlbRoman;
    private JButton jbtPlusOne, jbtMinusOne, jbtPlusTen, jbtMinusTen, jbtTimesTen, jbtDivideTen, jbtSetValue;
    private JTextField jtfNewValue;
    
    private NumberStringTest() {
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
    }
    
    private void createAndShowGUI() {
        JFrame frame = new JFrame("Numbers");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        Container pane = frame.getContentPane();
        pane.setLayout(new GridBagLayout());
        int row = 0;
        
        int WEST = GridBagConstraints.WEST;
        int CENTER = GridBagConstraints.CENTER;
        int EAST = GridBagConstraints.EAST;
        
        jlbDigits = new JLabel();
        addToPane(pane, jlbDigits, 0, row, 2, 1, CENTER);
        row++;
        
        jlbEnglish = new JLabel();
        addToPane(pane, jlbEnglish, 0, row, 2, 1, CENTER);
        row++;
        
        jlbFrench = new JLabel();
        addToPane(pane, jlbFrench, 0, row, 2, 1, CENTER);
        row++;
        
        jlbGerman = new JLabel();
        addToPane(pane, jlbGerman, 0, row, 2, 1, CENTER);
        row++;
        
        jlbRoman = new JLabel();
        addToPane(pane, jlbRoman, 0, row, 2, 1, CENTER);
        row++;
        
        jbtMinusOne = new JButton("-1");
        jbtMinusOne.addActionListener(this);
        addToPane(pane, jbtMinusOne, 0, row, 1, 1, EAST);
        
        jbtPlusOne = new JButton("+1");
        jbtPlusOne.addActionListener(this);
        addToPane(pane, jbtPlusOne, 1, row, 1, 1, WEST);
        row++;
        
        jbtMinusTen = new JButton("-10");
        jbtMinusTen.addActionListener(this);
        addToPane(pane, jbtMinusTen, 0, row, 1, 1, EAST);
        
        jbtPlusTen = new JButton("+10");
        jbtPlusTen.addActionListener(this);
        addToPane(pane, jbtPlusTen, 1, row, 1, 1, WEST);
        row++;
        
        jbtDivideTen = new JButton("/10");
        jbtDivideTen.addActionListener(this);
        addToPane(pane, jbtDivideTen, 0, row, 1, 1, EAST);
        
        jbtTimesTen = new JButton("*10");
        jbtTimesTen.addActionListener(this);
        addToPane(pane, jbtTimesTen, 1, row, 1, 1, WEST);
        row++;
        
        jtfNewValue = new JTextField("123", 10);
        addToPane(pane, jtfNewValue, 0, row, 1, 1, EAST);
        
        jbtSetValue = new JButton("Set");
        jbtSetValue.addActionListener(this);
        addToPane(pane, jbtSetValue, 1, row, 1, 1, WEST);
        row++;
        
        frame.pack();
        frame.setVisible(true);
        actionPerformed(null);
    }
    
    private void addToPane(Container pane, JComponent comp, int x, int y, int w, int h, int anchor) {
        GridBagConstraints gBC = new GridBagConstraints();
        gBC.insets = new Insets(5, 5, 5, 5);
        gBC.gridx = x;
        gBC.gridy = y;
        gBC.gridwidth = w;
        gBC.gridheight = h;
        gBC.anchor = anchor;
        gBC.weightx = 1;
        gBC.weighty = 0;
        pane.add(comp, gBC);
    }
    
    public void actionPerformed (ActionEvent ae) {
        NumberString ns = new NumberString();
        if (ae != null) {
            if (ae.getSource() == jbtPlusOne) {
                n++;
            } else if (ae.getSource() == jbtMinusOne) {
                n--;
            } else if (ae.getSource() == jbtPlusTen) {
                n += 10;
            } else if (ae.getSource() == jbtMinusTen) {
                n -= 10;
            } else if (ae.getSource() == jbtTimesTen) {
                n *= 10;
            } else if (ae.getSource() == jbtDivideTen) {
                n /= 10;
            } else if (ae.getSource() == jbtSetValue) {
                n = ns.longFromDigits(jtfNewValue.getText());
            }
        }
        jlbDigits.setText(ns.toDigits(n));
        jlbEnglish.setText(ns.toWords(n));
        ns.setLanguage(NumberString.FRENCH);
        jlbFrench.setText(ns.toWords(n));
        ns.setLanguage(NumberString.GERMAN);
        jlbGerman.setText(ns.toWords(n));
        jlbRoman.setText(n > 0 && n < 10000 ? ns.toRomanNumerals(n) : "-");
    }
    
    /**
     * Runs the test.
     * 
     * @param args the arguments, which are ignored
     */
    public static void main(String[] args) {
        if (args.length > 0 && (args[0].equals("-h") || args[0].equals("--help"))) {
            System.out.println("smithers.extras.NumberStringTest -- graphical test program for NumberString.");
            return;
        }
        new NumberStringTest();
    }
    
}
