// Copyright 2009 Christopher Smith
// 
// This file is part of smithers.extras.
// 
// smithers.extras is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.extras is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
// or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.extras. If not, see <http://www.gnu.org/licenses/>.

package smithers.extras;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.HashMap;
import javax.imageio.ImageIO;
import smithers.util.Launcher;

class TileRipper {
    
    private static PrintWriter out;
    private static BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
    private static ImagePanel panel;
    private static HashMap<HashableIntegerArray, String> tileCache = new HashMap<HashableIntegerArray, String>();
    
    private static class HashableIntegerArray {
        int[] array;
        int hash;
        public HashableIntegerArray(int[] array) {
            this.array = array;
            hash = -array.length;
            for (int i=0; i<array.length; i++)
                hash += array[i] * (2*i+1);
        }
        public boolean equals(Object x) {
            if (!(x instanceof HashableIntegerArray)) return false;
            HashableIntegerArray that = (HashableIntegerArray) x;
            if (this.array.length != that.array.length) return false;
            for (int i=0; i<array.length; i++)
                if (this.array[i] != that.array[i]) return false;
            return true;
        }
        public int hashCode() {
            return hash;
        }
    }
    
    private static class ImagePanel extends Component {
        Image img = null;
        private void setImage(Image newImage) {
            img = newImage;
            repaint();
        }
        public void paint(Graphics g) {
            Rectangle r = getBounds();
            if (img==null)
                g.clearRect(0, 0, r.width, r.height);
            else
                g.drawImage(img, 0, 0, r.width, r.height, 0, 0, img.getWidth(null), img.getHeight(null), null);
        }
    }
    
    private TileRipper() {}
    
    private static void doThing(BufferedImage img, int b, boolean swap) throws IOException {
        StringBuilder sb = new StringBuilder();
        int[] tile;
        String str;
        boolean isBlank;
        HashableIntegerArray hia;
        for (int x=0; x<(swap?img.getWidth():img.getHeight()); x+=b) {
            sb.delete(0, sb.length());
            for (int y=0; y<(swap?img.getHeight():img.getWidth()); y+=b) {
                tile = img.getRGB(swap?x:y, swap?y:x, b, b, null, 0, b);
                isBlank = true;
                for (int i=1; isBlank&&i<b*b; i++)
                    if (tile[i]!=tile[1]) isBlank=false;
                if (isBlank) continue;
                
                hia = new HashableIntegerArray(tile);
                if (tileCache.containsKey(hia)) {
                    str = tileCache.get(hia);
                } else {
                    panel.setImage(img.getSubimage(swap?x:y, swap?y:x, b, b));
                    System.out.println("Please recognise tile.");
                    str = in.readLine();
                    tileCache.put(hia, str);
                }
                
                if (sb.length() > 0) sb.append(' ');
                sb.append(str);
            }
            out.println(sb.toString());
        }
    }
    
    public static void main(String[] args) throws IOException {
        int b;
        boolean swap = false;
        BufferedImage img;
        panel = new ImagePanel();
        try {
            out = new PrintWriter(new BufferedWriter(new FileWriter(args[0])));
            b = Integer.parseInt(args[1]);
        } catch (Exception e) {
            System.out.println("Usage: TileRipper outputFile blockSize [+ | - | inputFile]...");
            System.exit(1);
            return;
        }
        Launcher.launch(panel, "TileRipper", b, b);
        for (int i=2; i<args.length; i++) {
            if (args[i].equals("-")) swap = true;
            else if (args[i].equals("+")) swap = false;
            else {
                out.println(args[i]);
                try {
                    img = ImageIO.read(new File(args[i]));
                } catch (Exception e) {
                    out.println("ERROR: Could not open image");
                    continue;
                }
                if (img.getWidth()%b!=0 || img.getHeight()%b!=0) {
                    out.println("ERROR: Image is not correctly sized");
                    continue;
                }
                out.println((swap?"- ":"+ ") + img.getWidth()/b + " " + img.getHeight()/b);
                doThing(img, b, swap);
            }
        }
        out.flush();
        System.out.println("Done now, please close window.");
    }
    
}
