// Copyright 2009 Christopher Smith
// 
// This file is part of smithers.svg.
// 
// smithers.svg is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by the Free
// Software Foundation, either version 3 of the License, or (at your option)
// any later version.
// 
// smithers.svg is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
// 
// You should have received a copy of the GNU General Public License along with
// smithers.svg. If not, see <http://www.gnu.org/licenses/>.

package smithers.svg;

import java.io.*;
import java.util.*;

/**
 * Represents a Scalable Vector Graphics (SVG) image.
 */
public class SVG {
    
    public List<SVGElement> elements = new LinkedList<SVGElement>();
    public String preamble, postamble;
    
    /**
     * Creates a new SVG image.
     * 
     * @param width  the width of the image
     * @param height the height of the image
     */
    public SVG(String width, String height) {
        preamble = "<?xml version=\"1.0\" standalone=\"no\"?>\n"
                   + "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\" \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">"
                   + "\n<svg width=\"" + width + "\" height=\"" + height + "\" version=\"1.1\" xmlns=\"http://www.w3.org/2000/svg\">";
        postamble = "</svg>";
    }
    
    /**
     * Adds a new element to this SVG image.
     * 
     * @param e the element to add
     */
    public void add(SVGElement e) {
        elements.add(e);
    }
    
    /**
     * Writes this SVG image to a file.
     * 
     * @param f the file
     */
    public void write(File f) throws IOException {
        if (f.exists()) f.delete();
        PrintWriter out = new PrintWriter(new BufferedWriter(new FileWriter(f)));
        
        out.println(preamble);
        for(SVGElement e : elements) {
            out.println(e.getCode().replace("\n","\n  "));
        }
        out.println(postamble);
        out.close();
    }
    
    /**
     * IN-DEV! Reads an SVG image from a file.
     * 
     * @param f the file
     * @return the SVG image
     */
    private SVG read(File f) throws IOException {
        BufferedReader inputStream = new BufferedReader(new FileReader(f));
        return new SVG("1", "1");
    }
    
}
